package com.flyfish.framework.validation.validators;

import com.flyfish.framework.validation.annotations.IdCard;

import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import java.util.regex.Pattern;

/**
 * 身份证验证，使用正则
 *
 * @author wangyu
 */
public class IdCardValidator implements ConstraintValidator<IdCard, String> {

    private static final Pattern PATTERN = Pattern.compile(
            "^[1-9]\\d{5}[1-9]\\d{3}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}([0-9]|X)$"
    );

    /**
     * Implements the validation logic.
     * The state of {@code value} must not be altered.
     * <p>
     * This method can be accessed concurrently, thread-safety must be ensured
     * by the implementation.
     *
     * @param value   object to validate
     * @param context context in which the constraint is evaluated
     * @return {@code false} if {@code value} does not pass the constraint
     */
    @Override
    public boolean isValid(String value, ConstraintValidatorContext context) {
        return null == value || PATTERN.matcher(value).matches();
    }
}
