package com.flyfish.framework.validation.validators;

import com.flyfish.framework.constant.Frameworks;
import com.flyfish.framework.validation.annotations.Password;

import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import java.util.regex.Pattern;

/**
 * 身份证验证，使用正则
 *
 * @author wangyu
 */
public class PasswordValidator implements ConstraintValidator<Password, String> {

    public static final String PATTERN = "^(?![a-zA-z]+$)(?!\\d+$)(?![,.!?~`_+=@#$%^&*;<>':]+$)(?![a-zA-z\\d]+$)(?![a-zA-z,.!?~`_+=@#$%^&*;<>':]+$)(?![\\d,.!?~`_+=@#$%^&*;<>':]+$)[a-zA-Z\\d,.!?~`_+=@#$%^&*;<>':]{12,18}$";
    public static final String SHORT_PATTERN = "^(?![a-zA-z]+$)(?!\\d+$)(?![,.!?~`_+=@#$%^&*;<>':]+$)(?![a-zA-z\\d]+$)(?![a-zA-z,.!?~`_+=@#$%^&*;<>':]+$)(?![\\d,.!?~`_+=@#$%^&*;<>':]+$)[a-zA-Z\\d,.!?~`_+=@#$%^&*;<>':]{8,18}$";

    /**
     * 此处传入密码明文判定密码可用性
     *
     * @param password 密码
     * @return 结果
     */
    public static boolean isValid(String password) {
        if (Frameworks.config.isShortPassword()) {
            return LazyHolder.shortRegex.matcher(password).matches();
        }
        return LazyHolder.normalRegex.matcher(password).matches();
    }

    /**
     * Implements the validation logic.
     * The state of {@code value} must not be altered.
     * <p>
     * This method can be accessed concurrently, thread-safety must be ensured
     * by the implementation.
     *
     * @param value   object to validate
     * @param context context in which the constraint is evaluated
     * @return {@code false} if {@code value} does not pass the constraint
     */
    @Override
    public boolean isValid(String value, ConstraintValidatorContext context) {
        return null == value || isValid(value);
    }

    private static class LazyHolder {
        private static final Pattern normalRegex = Pattern.compile(PATTERN);
        private static final Pattern shortRegex = Pattern.compile(SHORT_PATTERN);
    }
}
