package com.flyfish.framework.auditor.callback;

import com.flyfish.framework.auditor.BeanAuditor;
import com.flyfish.framework.domain.authorized.AuthorizedDomain;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.base.Domain;
import lombok.Setter;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mapping.callback.EntityCallback;

import static org.springframework.data.util.CastUtils.cast;

/**
 * 同步审查器回调，用于执行公共审查
 *
 * @author wangyu
 */
public class AuditorCallback implements EntityCallback<Domain> {

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<BeanAuditor<Domain>> commonAuditors;

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<BeanAuditor<AuditDomain>> operationAuditors;

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<BeanAuditor<AuthorizedDomain>> authorizeAuditors;

    /**
     * 执行实体类公共审查逻辑
     *
     * @param domain 实体信息
     * @return 结果
     */
    protected Domain audit(Domain domain) {
        // 执行auditor
        doAudit(domain, commonAuditors);
        if (domain instanceof AuditDomain) {
            doAudit(domain, operationAuditors);
        }
        if (domain instanceof AuthorizedDomain) {
            doAudit(domain, authorizeAuditors);
        }
        return domain;
    }

    /**
     * 执行审查逻辑
     *
     * @param domain   实体
     * @param auditors 审查器集合
     * @param <T>      实体泛型
     */
    private <T extends Domain> void doAudit(Domain domain, ObjectProvider<BeanAuditor<T>> auditors) {
        auditors.stream().forEach(auditor -> auditor.audit(cast(domain)));
    }
}
