package com.flyfish.framework.auditor.callback;

import com.flyfish.framework.auditor.ReactiveBeanAuditor;
import com.flyfish.framework.domain.authorized.AuthorizedDomain;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.base.Domain;
import lombok.Setter;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mapping.callback.EntityCallback;
import reactor.core.publisher.Mono;

import static org.springframework.data.util.CastUtils.cast;

/**
 * 支持审查的callback
 *
 * @author wangyu
 */
public abstract class ReactiveAuditorCallback implements EntityCallback<Domain> {

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<ReactiveBeanAuditor<Domain>> commonAuditors;

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<ReactiveBeanAuditor<AuditDomain>> operationAuditors;

    @Setter(onMethod_ = @Autowired)
    private ObjectProvider<ReactiveBeanAuditor<AuthorizedDomain>> authorizeAuditors;

    /**
     * 执行实体类公共审查逻辑
     *
     * @param domain 实体信息
     * @return 结果
     */
    protected Mono<Domain> audit(Domain domain) {
        // 执行auditor
        Mono<Domain> signal = doAudit(Mono.just(domain), commonAuditors);
        if (domain instanceof AuditDomain) {
            signal = doAudit(signal, operationAuditors);
        }
        if (domain instanceof AuthorizedDomain) {
            signal = doAudit(signal, authorizeAuditors);
        }
        return signal;
    }

    /**
     * 执行审查逻辑
     *
     * @param source   来源信号
     * @param auditors 审查器集合
     * @param <T>      实体泛型
     * @return 结果
     */
    private <T extends Domain> Mono<Domain> doAudit(Mono<Domain> source, ObjectProvider<ReactiveBeanAuditor<T>> auditors) {
        return auditors.stream().reduce(source, (r, auditor) -> r.flatMap(domain -> auditor.audit(cast(domain))), (a, b) -> a);
    }
}
