package com.flyfish.framework.query;

import com.flyfish.framework.domain.base.Domain;
import lombok.Data;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

import java.util.List;
import java.util.function.Function;

/**
 * 抽象查询实体
 *
 * @author wangyu
 */
@Data
public class Query {

    // 查询定义
    private QueryDefinition definition;
    // 查询用例
    private Example<?> example;
    // 排序
    private Sort sort;
    // 分页
    private Pageable pageable;
    // 字段列表
    private List<String> fields;

    /**
     * 初始化一个查询
     *
     * @param definition 查询定义
     * @return 结果
     */
    public static Query query(QueryDefinition definition) {
        Query query = new Query();
        query.setDefinition(definition);
        return query;
    }

    /**
     * 创建一个空查询
     *
     * @return 结果
     */
    public static Query empty() {
        return new Query();
    }

    /**
     * 指定选择的字段
     *
     * @param columns 字段们
     * @return 当前自身
     */
    public Query select(String... columns) {
        return this;
    }

    /**
     * 指定选择的字段
     *
     * @param getters 字段们
     * @return 当前自身
     */
    public <T extends Domain> Query select(Function<T, ?>... getters) {
        return this;
    }
}
