package com.flyfish.framework.annotations;

import java.lang.annotation.*;

/**
 * 值生成策略
 * 值生成只发生在表单初始化阶段
 * 对于父类使用的场景，如果子类进行了属性覆盖，但未指定生成策略，父类策略会被忽略
 *
 * @author wangyu
 */
@Target({ElementType.ANNOTATION_TYPE, ElementType.FIELD})
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface Generation {

    /**
     * 生成策略
     *
     * @return 结果
     */
    Strategy value() default Strategy.NONE;

    /**
     * 来自某个字段
     *
     * @return 结果
     */
    String[] from() default {};

    /**
     * 模板，支持生成模板
     *
     * @return 结果
     */
    String template() default "";

    /**
     * 代码生成策略
     */
    enum Strategy {

        NONE, NAME, CODE, UUID, TIMESTAMP, TRANSACTION
    }
}
