package com.flyfish.framework.context;

import com.flyfish.framework.context.interceptor.ViewModelInterceptor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * 全局上下文
 *
 * @author wangyu
 * 支持插件式增加处理器
 */
@Component
public class ViewModelContext {

    // 唯一实例
    private static ViewModelContext INSTANCE;

    // 拦截器们
    private final List<ViewModelInterceptor> interceptors;

    /**
     * 通过构造器注入
     */
    public ViewModelContext(List<ViewModelInterceptor> interceptors) {
        this.interceptors = interceptors;
        INSTANCE = this;
    }

    /**
     * 全局对象拦截器
     *
     * @param value 对象值
     * @param <T>   泛型
     * @return 结果
     */
    public static <T> Mono<T> intercept(T value) {
        if (null != INSTANCE) {
            List<ViewModelInterceptor> interceptors = INSTANCE.interceptors;
            if (CollectionUtils.isNotEmpty(interceptors)) {
                for (ViewModelInterceptor interceptor : interceptors) {
                    if (interceptor.supports(value)) {
                        return interceptor.intercept(value);
                    }
                }
            }
        }
        return Mono.just(value);
    }
}
