package com.flyfish.framework.domain.base;

import com.flyfish.framework.annotations.Property;
import lombok.Getter;
import lombok.Setter;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.relational.core.mapping.Column;

import java.time.LocalDateTime;

/**
 * 需要审查的domain（修改和更新指定人）
 *
 * @author wangyu
 */
@Getter
@Setter
public abstract class AuditDomain extends Domain {

    /**
     * 创建日期
     */
    @CreatedDate
    @Property(value = "创建日期", readonly = true)
    @Column("create_time")
    protected LocalDateTime createTime;

    /**
     * 修改日期
     */
    @LastModifiedDate
    @Property(value = "更新日期", readonly = true)
    @Column("modify_time")
    protected LocalDateTime modifyTime;

    /**
     * 创建者
     */
    @CreatedBy
    @Property(value = "创建人", readonly = true)
    protected String creator;

    /**
     * 创建人id
     */
    @Property(readonly = true)
    @Column("creator_id")
    protected String creatorId;

    /**
     * 修改者
     */
    @LastModifiedBy
    @Property(value = "更新人", readonly = true)
    protected String modifier;

    /**
     * 修改人id
     */
    @Property(readonly = true)
    @Column("modifier_id")
    protected String modifierId;
}
