package com.flyfish.framework.r2dbc.config.convert;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.flyfish.framework.utils.JacksonUtil;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.core.convert.TypeDescriptor;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.GenericConverter;
import org.springframework.lang.NonNull;

import java.nio.ByteBuffer;
import java.util.*;

/**
 * 额外的转换器
 * 支持各种奇奇怪怪的转换
 *
 * @author wangyu
 */
public final class AdditionalConverters {

    public static List<Object> ADDING_CONVERTERS = Arrays.asList(
            ByteBufferToBooleanConverter.INSTANCE,
            StringToCollectionConverter.INSTANCE
    );

    public enum StringToCollectionConverter implements GenericConverter {
        INSTANCE;

        @Override
        public Set<ConvertiblePair> getConvertibleTypes() {
            return Collections.singleton(new ConvertiblePair(String.class, Collection.class));
        }

        @Override
        public Object convert(Object source, @NonNull TypeDescriptor sourceType, @NonNull TypeDescriptor targetType) {
            if (ObjectUtils.isEmpty(source)) {
                return null;
            }
            String json = (String) source;
            TypeDescriptor elemType = targetType.getElementTypeDescriptor();
            if (null == elemType) {
                // 元素为空，都是模糊类型，直接尝试转换
                return JacksonUtil.fromJson(json);
            } else {
                JavaType jsonType = TypeFactory.defaultInstance().constructParametricType(targetType.getType(), elemType.getType());
                return JacksonUtil.fromJson((String) source, jsonType);
            }
        }
    }

    public enum ByteBufferToBooleanConverter implements Converter<ByteBuffer, Boolean> {

        INSTANCE;

        /**
         * Convert the source object of type {@code S} to target type {@code T}.
         *
         * @param source the source object to convert, which must be an instance of {@code S} (never {@code null})
         * @return the converted object, which must be an instance of {@code T} (potentially {@code null})
         * @throws IllegalArgumentException if the source cannot be converted to the desired target type
         */
        @Override
        public Boolean convert(ByteBuffer source) {
            return source.get() == 1;
        }
    }

}
