package com.flyfish.framework.r2dbc.config.dialect;

import io.r2dbc.spi.ConnectionFactory;
import io.r2dbc.spi.ConnectionFactoryMetadata;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.data.r2dbc.dialect.DialectResolver;
import org.springframework.data.r2dbc.dialect.R2dbcDialect;
import org.springframework.lang.NonNull;

import java.util.Map;
import java.util.Optional;

/**
 * 高级别名提供者
 *
 * @author wangyu
 */
@Order(Ordered.HIGHEST_PRECEDENCE)
public class AdvanceDialectProvider implements DialectResolver.R2dbcDialectProvider {

    private final Map<String, R2dbcDialect> DIALECTS = Map.of(
            "MySQL", AdvanceMysqlDialect.INSTANCE
    );

    /**
     * Returns a {@link R2dbcDialect} for a {@link ConnectionFactory}.
     *
     * @param connectionFactory the connection factory to be used with the {@link R2dbcDialect}.
     * @return {@link Optional} containing the {@link R2dbcDialect} if the {@link DialectResolver.R2dbcDialectProvider} can provide a
     * dialect object, otherwise {@link Optional#empty()}.
     */
    @Override
    @NonNull
    public Optional<R2dbcDialect> getDialect(ConnectionFactory connectionFactory) {
        ConnectionFactoryMetadata metadata = connectionFactory.getMetadata();
        R2dbcDialect r2dbcDialect = DIALECTS.get(metadata.getName());
        return Optional.ofNullable(r2dbcDialect);
    }
}
