package com.flyfish.framework.file.service;


import com.flyfish.framework.file.config.UploadConfiguration;
import com.flyfish.framework.file.utils.SystemUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.codec.multipart.FilePart;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.function.Supplier;

/**
 * 文件服务
 *
 * @author wangyu
 */
@Service
public class FileService {

    @Resource
    private UploadConfiguration uploadConfiguration;

    private List<Supplier<String>> generator = Arrays.asList(
            SystemUtils::generateDatePath
    );

    /**
     * 保存本地文件
     *
     * @return 保存到本地，并返回相对路径
     */
    public Mono<String> saveLocal(FilePart part) {
        String localPath = uploadConfiguration.getLocalPath();
        String extend = StringUtils.substringAfterLast(part.filename(), ".");
        String path = uploadConfiguration.path(generator) + "/" + UUID.randomUUID().toString() + "." + extend;
        String fullPath = localPath + path;
        SystemUtils.testPath(fullPath);
        return part.transferTo(Paths.get(fullPath))
                .then(Mono.just(path));
    }

    /**
     * 根据指定目录保存文件
     *
     * @param part 文件
     * @param path 路径
     * @return 结果
     */
    public Mono<String> saveLocalPath(FilePart part, String path) {
        SystemUtils.testPath(path);
        return part.transferTo(Paths.get(path))
                .then(Mono.just(path));
    }
}
