package com.flyfish.framework.user.service.impl;

import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.user.service.DepartmentService;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.r2dbc.core.R2dbcEntityOperations;
import org.springframework.data.relational.core.query.Criteria;
import org.springframework.data.relational.core.query.Query;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;

import static com.flyfish.framework.user.service.impl.DepartmentServiceHelper.convertQo;

/**
 * 部门服务
 *
 * @author wangyu
 */
@Service
@Slf4j
@ConditionalOnProperty("spring.r2dbc.url")
public class R2dbcDepartmentServiceImpl extends BaseReactiveServiceImpl<Department> implements DepartmentService {

    @Resource
    private R2dbcEntityOperations r2dbcEntityOperations;

    /**
     * 获取子部门列表
     *
     * @return 结果
     */
    public Mono<Set<String>> getSubCodes(Set<String> parents) {
        Query query = Query.query(Criteria.where("parentIds").in(parents));
        query.columns("id");
        return r2dbcEntityOperations.select(query, Department.class).map(Department::getId).collect(Collectors.toSet());
    }

    /**
     * 查询列表
     *
     * @param query 查询
     * @return 结果
     */
    @Override
    public Flux<Department> getList(Qo<Department> query) {
        return convertQo(query, this::getMinDepthInIds).flatMapMany(super::getList);
    }

    /**
     * 在一批编码中找到深度最小的值
     *
     * @param codes 编码们
     * @return 结果
     */
    private Mono<Integer> getMinDepthInIds(Collection<String> codes) {
        return repository.findAllById(codes)
                .map(Department::getDepth)
                .reduce(Integer::min);
    }


}
