package com.flyfish.framework.web;

import com.flyfish.framework.configuration.boot.FlyfishApplication;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.domain.po.Role;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.RoleType;
import com.flyfish.framework.r2dbc.config.R2dbcDataConfig;
import com.flyfish.framework.user.annotations.EnableAutoSecurity;
import com.flyfish.framework.user.repository.UserRepository;
import com.flyfish.framework.utils.JacksonUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Arrays;

@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = FlyfishUserTest.class)
@FlyfishApplication(scanBasePackages = "com.flyfish.framework")
@Import(R2dbcDataConfig.class)
@EnableAutoSecurity
@Slf4j
public class FlyfishUserTest {

    @Resource
    private UserRepository userRepository;

    @Test
    public void test() {
        User user = userRepository.findByUsername("admin")
                .doOnNext(this::log)
                .block();
        // 保存查询部门
        Department department = new Department();
        department.setName("测试部门");
        department.setFullName("完整部门");
        department.setCode("test");
        user.setDepartments(Arrays.asList(department));
        // 保存角色
        Role role = new Role();
        role.setCode("default");
        role.setAuthorities(Arrays.asList(Role.Authority.EDIT));
        role.setName("默认角色");
        role.setType(RoleType.PC);
        role.setAdmin(true);
        role.setDescription("测试角色");
        user.setRoles(Arrays.asList(role));
        // 完成
        userRepository.save(user)
                .doOnNext(this::log)
                .block();

    }

    @Test
    public void testNested() {
        // 查询role
        userRepository.findByUsername("admin")
                .doOnNext(this::log)
                .block();
    }

    private void log(Object data) {
        log.info(JacksonUtil.toPrettyJson(data));
    }
}
