package com.flyfish.framework.configuration.jwt;

import com.flyfish.framework.service.FlyfishUserDetailsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextImpl;
import org.springframework.security.web.server.context.ServerSecurityContextRepository;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.net.URI;

@Slf4j
public class JwtSecurityContextRepository implements ServerSecurityContextRepository {

    @Resource
    private FlyfishUserDetailsService userDetailsService;
    @Resource
    private TokenProvider tokenProvider;


    @Override
    public Mono<SecurityContext> load(ServerWebExchange serverWebExchange) {
        ServerHttpRequest request = serverWebExchange.getRequest();
        String jwt = tokenProvider.retrieveToken(serverWebExchange).orElse(null);
        URI requestURI = request.getURI();
        // 存在jwt时，校验jwt。redis也需要判断是否在黑名单
        return tokenProvider.validateToken(jwt).flatMap(userId -> {
            log.debug("set Authentication to security context for '{}', uri: {}", userId, requestURI);
            return userDetailsService.findByUsername(userId)
                    .map(userDetails -> {
                        SecurityContextImpl securityContext = new SecurityContextImpl();
                        Authentication authentication = new UsernamePasswordAuthenticationToken(userDetails, userDetails.getPassword(),
                                userDetails.getAuthorities());
                        securityContext.setAuthentication(authentication);
                        return securityContext;
                    });
        });
    }

    /**
     * 只有登录会调用这个
     *
     * @param webExchange     交换信息
     * @param securityContext 上下文
     * @return 结果
     */
    @Override
    public Mono<Void> save(ServerWebExchange webExchange, SecurityContext securityContext) {
        if (null == securityContext) {
            return tokenProvider.removeToken(webExchange);
        } else {
            // 添加jwtToken
            tokenProvider.addToken(webExchange, securityContext.getAuthentication());
        }
        return Mono.empty();
    }
}
